@@ -16,7 +16,7 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- * 
+ *
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  */
 
@@ -51,7 +51,6 @@ struct _EmpathyTpContactListClass {
 GType                  empathy_tp_contact_list_get_type    (void) G_GNUC_CONST;
 EmpathyTpContactList * empathy_tp_contact_list_new         (TpConnection         *connection);
 TpConnection *         empathy_tp_contact_list_get_connection (EmpathyTpContactList *list);
-gboolean               empathy_tp_contact_list_can_add     (EmpathyTpContactList *list);
 void                   empathy_tp_contact_list_remove_all  (EmpathyTpContactList *list);
 
 G_END_DECLS
