@@ -124,7 +124,7 @@ typedef struct {
 	GList                  *actions_connected;
 
 	/* The idle event source to migrate butterfly's logs */
-	guint butterfly_log_migration_contact_added_id;
+	guint butterfly_log_migration_members_changed_id;
 } EmpathyMainWindow;
 
 static EmpathyMainWindow *main_window = NULL;
@@ -324,7 +324,7 @@ main_window_row_activated_cb (EmpathyContactListView *view,
 
 static void
 main_window_remove_error (EmpathyMainWindow *window,
-			  TpAccount *account)
+			  TpAccount         *account)
 {
 	GtkWidget *error_widget;
 
@@ -336,15 +336,15 @@ main_window_remove_error (EmpathyMainWindow *window,
 }
 
 static void
-main_window_account_disabled_cb (TpAccountManager *manager,
-				 TpAccount *account,
+main_window_account_disabled_cb (TpAccountManager  *manager,
+				 TpAccount         *account,
 				 EmpathyMainWindow *window)
 {
 	main_window_remove_error (window, account);
 }
 
 static void
-main_window_error_retry_clicked_cb (GtkButton *button,
+main_window_error_retry_clicked_cb (GtkButton         *button,
 				    EmpathyMainWindow *window)
 {
 	TpAccount *account;
@@ -356,7 +356,7 @@ main_window_error_retry_clicked_cb (GtkButton *button,
 }
 
 static void
-main_window_error_edit_clicked_cb (GtkButton *button,
+main_window_error_edit_clicked_cb (GtkButton         *button,
 				   EmpathyMainWindow *window)
 {
 	TpAccount *account;
@@ -371,7 +371,7 @@ main_window_error_edit_clicked_cb (GtkButton *button,
 }
 
 static void
-main_window_error_close_clicked_cb (GtkButton *button,
+main_window_error_close_clicked_cb (GtkButton         *button,
 				    EmpathyMainWindow *window)
 {
 	TpAccount *account;
@@ -523,11 +523,11 @@ main_window_update_status (EmpathyMainWindow *window)
 
 static void
 main_window_connection_changed_cb (TpAccount  *account,
-                                   guint       old_status,
-                                   guint       current,
-                                   guint       reason,
-                                   gchar      *dbus_error_name,
-                                   GHashTable *details,
+				   guint       old_status,
+				   guint       current,
+				   guint       reason,
+				   gchar      *dbus_error_name,
+				   GHashTable *details,
 				   EmpathyMainWindow *window)
 {
 	main_window_update_status (window);
@@ -555,49 +555,6 @@ main_window_connection_changed_cb (TpAccount  *account,
 	}
 }
 
-static void
-main_window_contact_presence_changed_cb (EmpathyContactMonitor *monitor,
-					 EmpathyContact *contact,
-					 TpConnectionPresenceType current,
-					 TpConnectionPresenceType previous,
-					 EmpathyMainWindow *window)
-{
-  TpAccount *account;
-  gboolean should_play = FALSE;
-  EmpathyIdle *idle;
-
-  account = empathy_contact_get_account (contact);
-  idle = empathy_idle_dup_singleton ();
-
-  should_play = !empathy_idle_account_is_just_connected (idle, account);
-
-  if (!should_play)
-    goto out;
-
-  if (tp_connection_presence_type_cmp_availability (previous,
-     TP_CONNECTION_PRESENCE_TYPE_OFFLINE) > 0)
-    {
-      /* contact was online */
-      if (tp_connection_presence_type_cmp_availability (current,
-          TP_CONNECTION_PRESENCE_TYPE_OFFLINE) <= 0)
-        /* someone is logging off */
-        empathy_sound_play (GTK_WIDGET (window->window),
-          EMPATHY_SOUND_CONTACT_DISCONNECTED);
-    }
-  else
-    {
-      /* contact was offline */
-      if (tp_connection_presence_type_cmp_availability (current,
-          TP_CONNECTION_PRESENCE_TYPE_OFFLINE) > 0)
-        /* someone is logging in */
-        empathy_sound_play (GTK_WIDGET (window->window),
-          EMPATHY_SOUND_CONTACT_CONNECTED);
-    }
-
-out:
-  g_object_unref (idle);
-}
-
 static void
 main_window_accels_load (void)
 {
@@ -667,9 +624,9 @@ main_window_destroy_cb (GtkWidget         *widget,
 }
 
 static gboolean
-main_window_key_press_event_cb  (GtkWidget *window,
+main_window_key_press_event_cb  (GtkWidget   *window,
 				 GdkEventKey *event,
-				 gpointer user_data)
+				 gpointer     user_data)
 {
 	EmpathyChatManager *chat_manager;
 
@@ -706,7 +663,7 @@ main_window_chat_new_message_cb (GtkAction         *action,
 
 static void
 main_window_chat_new_call_cb (GtkAction         *action,
-				 EmpathyMainWindow *window)
+			      EmpathyMainWindow *window)
 {
 	empathy_new_call_dialog_show (GTK_WINDOW (window->window));
 }
@@ -801,8 +758,8 @@ main_window_view_sort_contacts_cb (GtkRadioAction    *action,
 }
 
 static void
-main_window_view_show_protocols_cb (GtkToggleAction *action,
-					EmpathyMainWindow *window)
+main_window_view_show_protocols_cb (GtkToggleAction   *action,
+				    EmpathyMainWindow *window)
 {
 	gboolean value;
 
@@ -843,8 +800,8 @@ main_window_view_contacts_list_size_cb (GtkRadioAction    *action,
 }
 
 static void main_window_notify_show_protocols_cb (EmpathyConf       *conf,
-					const gchar       *key,
-					EmpathyMainWindow *window)
+						  const gchar       *key,
+						  EmpathyMainWindow *window)
 {
 	gboolean show_protocols;
 
@@ -896,7 +853,8 @@ main_window_view_show_map_cb (GtkCheckMenuItem  *item,
 }
 
 static void
-main_window_favorite_chatroom_join (EmpathyChatroom *chatroom)
+join_chatroom (EmpathyChatroom *chatroom,
+	       gint64 timestamp)
 {
 	TpAccount      *account;
 	TpConnection   *connection;
@@ -904,17 +862,116 @@ main_window_favorite_chatroom_join (EmpathyChatroom *chatroom)
 
 	account = empathy_chatroom_get_account (chatroom);
 	connection = tp_account_get_connection (account);
+	g_assert (connection != NULL);
 	room = empathy_chatroom_get_room (chatroom);
 
-	if (connection != NULL) {
-		DEBUG ("Requesting channel for '%s'", room);
-		empathy_dispatcher_join_muc (connection, room,
-			gtk_get_current_event_time (), NULL, NULL);
-	}
+	DEBUG ("Requesting channel for '%s'", room);
+	empathy_dispatcher_join_muc (connection, room, timestamp, NULL, NULL);
+}
+
+typedef struct
+{
+	TpAccount *account;
+	EmpathyChatroom *chatroom;
+	gint64 timestamp;
+	glong sig_id;
+	guint timeout;
+} join_fav_account_sig_ctx;
+
+static join_fav_account_sig_ctx *
+join_fav_account_sig_ctx_new (TpAccount *account,
+			     EmpathyChatroom *chatroom,
+			      gint64 timestamp)
+{
+	join_fav_account_sig_ctx *ctx = g_slice_new0 (
+		join_fav_account_sig_ctx);
+
+	ctx->account = g_object_ref (account);
+	ctx->chatroom = g_object_ref (chatroom);
+	ctx->timestamp = timestamp;
+	return ctx;
 }
 
 static void
-main_window_favorite_chatroom_menu_activate_cb (GtkMenuItem    *menu_item,
+join_fav_account_sig_ctx_free (join_fav_account_sig_ctx *ctx)
+{
+	g_object_unref (ctx->account);
+	g_object_unref (ctx->chatroom);
+	g_slice_free (join_fav_account_sig_ctx, ctx);
+}
+
+static void
+account_status_changed_cb (TpAccount  *account,
+			   TpConnectionStatus old_status,
+			   TpConnectionStatus new_status,
+			   guint reason,
+			   gchar *dbus_error_name,
+			   GHashTable *details,
+			   gpointer user_data)
+{
+	join_fav_account_sig_ctx *ctx = user_data;
+
+	switch (new_status) {
+		case TP_CONNECTION_STATUS_DISCONNECTED:
+			/* Don't wait any longer */
+			goto finally;
+			break;
+
+		case TP_CONNECTION_STATUS_CONNECTING:
+			/* Wait a bit */
+			return;
+
+		case TP_CONNECTION_STATUS_CONNECTED:
+			/* We can join the room */
+			break;
+	}
+
+	join_chatroom (ctx->chatroom, ctx->timestamp);
+
+finally:
+	g_source_remove (ctx->timeout);
+	g_signal_handler_disconnect (account, ctx->sig_id);
+}
+
+#define JOIN_FAVORITE_TIMEOUT 5
+
+static gboolean
+join_favorite_timeout_cb (gpointer data)
+{
+	join_fav_account_sig_ctx *ctx = data;
+
+	/* stop waiting for joining the favorite room */
+	g_signal_handler_disconnect (ctx->account, ctx->sig_id);
+	return FALSE;
+}
+
+static void
+main_window_favorite_chatroom_join (EmpathyChatroom *chatroom)
+{
+	TpAccount      *account;
+
+	account = empathy_chatroom_get_account (chatroom);
+	if (tp_account_get_connection_status (account, NULL) !=
+					     TP_CONNECTION_STATUS_CONNECTED) {
+		join_fav_account_sig_ctx *ctx;
+
+		ctx = join_fav_account_sig_ctx_new (account, chatroom,
+			gtk_get_current_event_time ());
+
+		ctx->sig_id = g_signal_connect_data (account, "status-changed",
+			G_CALLBACK (account_status_changed_cb), ctx,
+			(GClosureNotify) join_fav_account_sig_ctx_free, 0);
+
+		ctx->timeout = g_timeout_add_seconds (JOIN_FAVORITE_TIMEOUT,
+			join_favorite_timeout_cb, ctx);
+		return;
+	}
+
+	join_chatroom (chatroom, gtk_get_current_event_time ());
+}
+
+static void
+main_window_favorite_chatroom_menu_activate_cb (GtkMenuItem     *menu_item,
 						EmpathyChatroom *chatroom)
 {
 	main_window_favorite_chatroom_join (chatroom);
@@ -922,7 +979,7 @@ main_window_favorite_chatroom_menu_activate_cb (GtkMenuItem    *menu_item,
 
 static void
 main_window_favorite_chatroom_menu_add (EmpathyMainWindow *window,
-					EmpathyChatroom    *chatroom)
+					EmpathyChatroom   *chatroom)
 {
 	GtkWidget   *menu_item;
 	const gchar *name;
@@ -948,7 +1005,7 @@ main_window_favorite_chatroom_menu_add (EmpathyMainWindow *window,
 static void
 main_window_favorite_chatroom_menu_added_cb (EmpathyChatroomManager *manager,
 					     EmpathyChatroom        *chatroom,
-					     EmpathyMainWindow     *window)
+					     EmpathyMainWindow      *window)
 {
 	main_window_favorite_chatroom_menu_add (window, chatroom);
 	gtk_widget_show (window->room_separator);
@@ -958,7 +1015,7 @@ main_window_favorite_chatroom_menu_added_cb (EmpathyChatroomManager *manager,
 static void
 main_window_favorite_chatroom_menu_removed_cb (EmpathyChatroomManager *manager,
 					       EmpathyChatroom        *chatroom,
-					       EmpathyMainWindow     *window)
+					       EmpathyMainWindow      *window)
 {
 	GtkWidget *menu_item;
 	GList *chatrooms;
@@ -1040,7 +1097,7 @@ main_window_room_manage_favorites_cb (GtkAction         *action,
 }
 
 static void
-main_window_edit_cb (GtkAction *action,
+main_window_edit_cb (GtkAction         *action,
 		     EmpathyMainWindow *window)
 {
 	GtkWidget *submenu;
@@ -1193,9 +1250,9 @@ main_window_account_removed_cb (TpAccountManager  *manager,
 }
 
 static void
-main_window_account_validity_changed_cb (TpAccountManager *manager,
-					 TpAccount *account,
-					 gboolean valid,
+main_window_account_validity_changed_cb (TpAccountManager  *manager,
+					 TpAccount         *account,
+					 gboolean           valid,
 					 EmpathyMainWindow *window)
 {
 	if (valid) {
@@ -1259,9 +1316,9 @@ empathy_main_window_get (void)
 }
 
 static void
-account_manager_prepared_cb (GObject *source_object,
+account_manager_prepared_cb (GObject      *source_object,
 			     GAsyncResult *result,
-			     gpointer user_data)
+			     gpointer      user_data)
 {
 	GList *accounts, *j;
 	TpAccountManager *manager = TP_ACCOUNT_MANAGER (source_object);
@@ -1300,14 +1357,21 @@ account_manager_prepared_cb (GObject *source_object,
 }
 
 static void
-main_window_contact_added_cb (EmpathyContactMonitor	*monitor,
-			      EmpathyContact		*contact,
-			      EmpathyMainWindow		*window)
+main_window_members_changed_cb (EmpathyContactList *list,
+				EmpathyContact     *contact,
+				EmpathyContact     *actor,
+				guint               reason,
+				gchar              *message,
+				gboolean            is_member,
+				EmpathyMainWindow  *window)
 {
+	if (!is_member)
+		return;
+
 	if (!empathy_migrate_butterfly_logs (contact)) {
-		g_signal_handler_disconnect (monitor,
-					     window->butterfly_log_migration_contact_added_id);
-		window->butterfly_log_migration_contact_added_id = 0;
+		g_signal_handler_disconnect (list,
+			window->butterfly_log_migration_members_changed_id);
+		window->butterfly_log_migration_members_changed_id = 0;
 	}
 }
 
@@ -1316,7 +1380,6 @@ empathy_main_window_show (void)
 {
 	EmpathyMainWindow        *window;
 	EmpathyContactList       *list_iface;
-	EmpathyContactMonitor    *monitor;
 	GtkBuilder               *gui;
 	EmpathyConf              *conf;
 	GtkWidget                *sw;
@@ -1449,15 +1512,13 @@ empathy_main_window_show (void)
 	window->throbber_tool_item = GTK_WIDGET (item);
 
 	list_iface = EMPATHY_CONTACT_LIST (empathy_contact_manager_dup_singleton ());
-	monitor = empathy_contact_list_get_monitor (list_iface);
 	window->list_store = empathy_contact_list_store_new (list_iface);
 	window->list_view = empathy_contact_list_view_new (window->list_store,
 							   EMPATHY_CONTACT_LIST_FEATURE_ALL,
 							   EMPATHY_CONTACT_FEATURE_ALL);
-	g_signal_connect (monitor, "contact-presence-changed",
-			  G_CALLBACK (main_window_contact_presence_changed_cb), window);
-	window->butterfly_log_migration_contact_added_id =  g_signal_connect (monitor, "contact-added",
-			  G_CALLBACK (main_window_contact_added_cb), window);
+	window->butterfly_log_migration_members_changed_id = g_signal_connect (
+		list_iface, "members-changed",
+		G_CALLBACK (main_window_members_changed_cb), window);
 	g_object_unref (list_iface);
 
 	gtk_widget_show (GTK_WIDGET (window->list_view));
